<?php

namespace Drupal\tapis_app_webform\Plugin\EntityReferenceSelection;

use Drupal\node\Plugin\EntityReferenceSelection\NodeSelection;

/**
 * Provides specific access control for the node entity type.
 *
 * This Plugin is used in the tapis_app and tapis_job modules
 * so that the "System" select field in an app's launch/restart/clone/etc.
 * webforms only shows systems that are listed
 * in the app's allowed systems list.
 *
 * @EntityReferenceSelection(
 *   id = "default:system_node_by_ids",
 *   label = @Translation("System node by ids selection"),
 *   entity_types = {"node"},
 *   group = "default",
 *   weight = 10
 * )
 */
class SystemNodeByIdsSelection extends NodeSelection {

  /**
   * {@inheritdoc}
   */
  protected function buildEntityQuery($match = NULL, $match_operator = 'CONTAINS') {
    $query = parent::buildEntityQuery($match, $match_operator);
    $allowed_system_ids = $this->getConfiguration()['allowed_system_ids'] ?? [];
    if (!$allowed_system_ids) {
      return $query;
    }
    $query->condition('nid', $allowed_system_ids, 'IN');
    return $query;
  }

}
