<?php

namespace Drupal\tapis_app_webform\Plugin\WebformElement;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformElement\Checkbox;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Provides a 'tapis_app_webform_checkbox' element.
 *
 * @WebformElement(
 *   id = "tapis_app_webform_checkbox",
 *   label = @Translation("Checkbox (OSP)"),
 *   description = @Translation("Provides a checkbox element for App forms."),
 *   category = @Translation("One Science Place (OSP)"),
 * )
 *
 * @see \Drupal\webform_example_element\Element\WebformExampleElement
 * @see \Drupal\webform\Plugin\WebformElementBase
 * @see \Drupal\webform\Plugin\WebformElementInterface
 * @see \Drupal\webform\Annotation\WebformElement
 */
class OSPWebformCheckbox extends Checkbox {

  /**
   * {@inheritdoc}
   */
  public function prepare(array &$element, WebformSubmissionInterface $webform_submission = NULL) {
    $element['#type'] = 'checkbox';
    parent::prepare($element, $webform_submission);
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    // Here you can define and alter a webform element's properties UI.
    // Form element property visibility and default values are defined via
    // ::defaultProperties.
    //
    // @see \Drupal\webform\Plugin\WebformElementBase::form
    // @see \Drupal\webform\Plugin\WebformElement\TextBase::form
    // Get element properties.
    $element_properties = $form_state->getValues() ?: $form_state->get('element_properties');

    // Add a details element to $form for the following fields:
    // - cmdPrefix, cmdSuffix.
    $form['element']['cmd'] = [
      '#type' => 'details',
      '#access' => TRUE,
      '#title' => $this->t('Command arguments'),
      '#open' => TRUE,
    ];

    $form['element']['cmd']['valueType'] = [
      '#type' => 'select',
      '#access' => TRUE,
      '#title' => $this->t('Type'),
      '#description' => $this->t('The type of this value'),
      '#options' => [
        'cli_arg' => $this->t('Command line argument'),
        'env_var' => $this->t('Environment variable'),
        'scheduler_option' => $this->t('Scheduler option'),
      ],
      '#default_value' => $element_properties['valueType'] ?? 'cli_arg',
    ];

    // Add a textfield element to $form['cmd'] for the cmdPrefix property.
    $form['element']['cmd']['cmdPrefix'] = [
      '#type' => 'textfield',
      '#access' => TRUE,
      '#title' => $this->t('Command prefix'),
      '#description' => $this->t('The prefix to be added to the command.'),
      '#default_value' => $element_properties['cmdPrefix'] ?? '',
    ];

    // Add a checkbox element to $form['cmd']
    // for the cmdPrefix_trailing_space property.
    $form['element']['cmd']['cmdPrefix_trailing_space'] = [
      '#type' => 'checkbox',
      '#access' => TRUE,
      '#title' => $this->t('Add trailing space to command prefix'),
      '#description' => $this->t('Add a trailing space to the command prefix.'),
      '#default_value' => $element_properties['cmdPrefix_trailing_space'] ?? TRUE,
    ];

    // Add a textfield element to $form['cmd'] for the cmdSuffix property.
    $form['element']['cmd']['cmdSuffix'] = [
      '#type' => 'textfield',
      '#access' => TRUE,
      '#title' => $this->t('Command suffix'),
      '#description' => $this->t('The suffix to be added to the command.'),
      '#default_value' => $element_properties['cmdSuffix'] ?? '',
    ];

    // Add a checkbox element to $form['cmd']
    // for the cmdSuffix_trailing_space property.
    $form['element']['cmd']['cmdSuffix_trailing_space'] = [
      '#type' => 'checkbox',
      '#access' => TRUE,
      '#title' => $this->t('Add trailing space to command suffix'),
      '#description' => $this->t('Add a trailing space to the command suffix.'),
      '#default_value' => $element_properties['cmdSuffix_trailing_space'] ?? TRUE,
    ];

    // Add a checkbox element to $form['cmd'] for the ignoreValue property.
    $form['element']['cmd']['ignoreValue'] = [
      '#type' => 'checkbox',
      '#access' => TRUE,
      '#title' => $this->t('Ignore value'),
      '#description' => $this->t('Ignore the value of this element when generating the command.'),
      '#default_value' => $element_properties['ignoreValue'] ?? FALSE,
    ];

    // Add a checkbox element to $form['cmd']
    // for the showOnlyOnRestart property.
    $form['element']['cmd']['showOnlyOnRestart'] = [
      '#type' => 'checkbox',
      '#access' => TRUE,
      '#title' => $this->t('Show only on restart'),
      '#description' => $this->t('Show this element only when the job is restarted.'),
      '#default_value' => $element_properties['showOnlyOnRestart'] ?? FALSE,
    ];

    // Add a check element to $form['cmd'] for the ignoreElementKey property.
    $form['element']['cmd']['ignoreElementKey'] = [
      '#type' => 'textfield',
      '#access' => TRUE,
      '#title' => $this->t('Ignore element key'),
      '#description' => $this->t('If checked, ignore the value of the element with this key when generating the command.'),
      '#default_value' => $element_properties['ignoreElementKey'] ?? '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function defineDefaultProperties() {
    // Here you define your webform element's default properties,
    // which can be inherited.
    //
    // @see \Drupal\webform\Plugin\WebformElementBase::defaultProperties
    // @see \Drupal\webform\Plugin\WebformElementBase::defaultBaseProperties
    return [
      'valueType' => 'cli_arg',
      'cmdPrefix' => '',
      'cmdPrefix_trailing_space' => TRUE,
      'cmdSuffix' => '',
      'cmdSuffix_trailing_space' => TRUE,
      'ignoreValue' => FALSE,
      'showOnlyOnRestart' => FALSE,
      'ignoreElementKey' => '',
    ] + parent::defineDefaultProperties();
  }

  /**
   * {@inheritdoc}
   */
  protected function defineTranslatableProperties() {
    return array_merge(
      parent::defineTranslatableProperties(),
      [
        'valueType',
        'cmdPrefix',
        'cmdPrefix_trailing_space',
        'cmdSuffix',
        'cmdSuffix_trailing_space',
        'ignoreValue',
        'showOnlyOnRestart',
        'ignoreElementKey',
      ]
    );
  }

}
