<?php

namespace Drupal\tapis_app_webform\Plugin;

use Drupal\webform\Plugin\WebformElementManager;

/**
 * Manages permissioned webform elements.
 *
 * @phpstan-ignore-next-line
 */
class WebformPermissionedElementManager extends WebformElementManager {

  /**
   * {@inheritdoc}
   */
  public function removeExcludeDefinitions(array $definitions) {
    $excluded = parent::removeExcludeDefinitions($definitions);
    $container_element_keys = [
      'container',
      'details',
      'fieldset',
      'webform_flexbox',
      'item',
      'webform_section',
      "table",
      "webform_table",
      "webform_table_row",
      "tableselect",
      "webform_tableselect_sort",
      "webform_table_sort",
    ];

    // Check if the current user has the
    // 'use native webform elements' permission
    // if not, filter out the elements in the $excluded array
    // that are NOT osp elements
    // also, check if the current user is an Drupal site admin user.
    // if so, do not filter out any elements
    // finally, still show the container elements
    // in addition to the osp elements.
    // @phpstan-ignore-next-line
    if (\Drupal::currentUser()->id() != 1 && !\Drupal::currentUser()->hasPermission('use native webform elements')) {
      $filtered = [];
      foreach ($excluded as $key => $value) {
        if (strpos($key, 'tapis_app_webform_') !== FALSE) {
          $filtered[$key] = $value;
        }
        elseif (in_array($key, $container_element_keys)) {
          $filtered[$key] = $value;
        }
      }
      return $filtered;
    }
    return $excluded;
  }

}
